"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DateInput = void 0;
var _react = _interopRequireWildcard(require("react"));
var _legacyDateFormat = require("@sanity/util/legacyDateFormat");
var _PatchEvent = _interopRequireWildcard(require("../../PatchEvent"));
var _CommonDateTimeInput = require("./CommonDateTimeInput");
var _excluded = ["type", "onChange"];
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
function _extends() { _extends = Object.assign ? Object.assign.bind() : function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }
function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }
function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }
// This is the format dates are stored on
var VALUE_FORMAT = 'YYYY-MM-DD';
// default to how they are stored
var DEFAULT_DATE_FORMAT = VALUE_FORMAT;
function parseOptions() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return {
    dateFormat: options.dateFormat || DEFAULT_DATE_FORMAT,
    calendarTodayLabel: options.calendarTodayLabel || 'Today'
  };
}
var deserialize = value => (0, _legacyDateFormat.parse)(value, VALUE_FORMAT);
var serialize = date => (0, _legacyDateFormat.format)(date, VALUE_FORMAT);
var DateInput = /*#__PURE__*/_react.default.forwardRef(function DateInput(props, forwardedRef) {
  var type = props.type,
    onChange = props.onChange,
    rest = _objectWithoutProperties(props, _excluded);
  var title = type.title,
    description = type.description,
    placeholder = type.placeholder;
  var _parseOptions = parseOptions(type.options),
    dateFormat = _parseOptions.dateFormat;
  var handleChange = (0, _react.useCallback)(nextDate => {
    onChange(_PatchEvent.default.from([nextDate === null ? (0, _PatchEvent.unset)() : (0, _PatchEvent.set)(nextDate)]));
  }, [onChange]);
  var formatInputValue = _react.default.useCallback(date => (0, _legacyDateFormat.format)(date, dateFormat), [dateFormat]);
  var parseInputValue = _react.default.useCallback(inputValue => (0, _legacyDateFormat.parse)(inputValue, dateFormat), [dateFormat]);
  return /*#__PURE__*/_react.default.createElement(_CommonDateTimeInput.CommonDateTimeInput, _extends({}, rest, {
    onChange: handleChange,
    ref: forwardedRef,
    selectTime: false,
    title: title,
    description: description,
    placeholder: placeholder,
    parseInputValue: parseInputValue,
    formatInputValue: formatInputValue,
    deserialize: deserialize,
    serialize: serialize
  }));
});
exports.DateInput = DateInput;